@@ -14,11 +14,12 @@
 
 import uuid
 
-from novaclient import api_versions
+from openstack import utils as sdk_utils
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
 
+from openstackclient.common import pagination
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 
@@ -27,56 +28,34 @@ class ListMigration(command.Lister):
     _description = _("""List server migrations""")
 
     def get_parser(self, prog_name):
-        parser = super(ListMigration, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
         parser.add_argument(
             '--server',
             metavar='<server>',
-            help=_(
-                'Filter migrations by server (name or ID)'
-            )
+            help=_('Filter migrations by server (name or ID)'),
         )
         parser.add_argument(
             '--host',
             metavar='<host>',
-            help=_(
-                'Filter migrations by source or destination host'
-            ),
+            help=_('Filter migrations by source or destination host'),
         )
         parser.add_argument(
             '--status',
             metavar='<status>',
-            help=_('Filter migrations by status')
+            help=_('Filter migrations by status'),
         )
         parser.add_argument(
             '--type',
             metavar='<type>',
             choices=[
-                'evacuation', 'live-migration', 'cold-migration', 'resize',
+                'evacuation',
+                'live-migration',
+                'cold-migration',
+                'resize',
             ],
             help=_('Filter migrations by type'),
         )
-        parser.add_argument(
-            '--marker',
-            metavar='<marker>',
-            help=_(
-                "The last migration of the previous page; displays list "
-                "of migrations after 'marker'. Note that the marker is "
-                "the migration UUID. "
-                "(supported with --os-compute-api-version 2.59 or above)"
-            ),
-        )
-        parser.add_argument(
-            '--limit',
-            metavar='<limit>',
-            type=int,
-            help=_(
-                "Maximum number of migrations to display. Note that there "
-                "is a configurable max limit on the server, and the limit "
-                "that is used will be the minimum of what is requested "
-                "here and what is configured in the server. "
-                "(supported with --os-compute-api-version 2.59 or above)"
-            ),
-        )
+        pagination.add_marker_pagination_option_to_parser(parser)
         parser.add_argument(
             '--changes-since',
             dest='changes_since',
@@ -121,31 +100,47 @@ class ListMigration(command.Lister):
 
     def print_migrations(self, parsed_args, compute_client, migrations):
         column_headers = [
-            'Source Node', 'Dest Node', 'Source Compute', 'Dest Compute',
-            'Dest Host', 'Status', 'Server UUID', 'Old Flavor', 'New Flavor',
-            'Created At', 'Updated At',
+            'Source Node',
+            'Dest Node',
+            'Source Compute',
+            'Dest Compute',
+            'Dest Host',
+            'Status',
+            'Server UUID',
+            'Old Flavor',
+            'New Flavor',
+            'Created At',
+            'Updated At',
         ]
 
         # Response fields coming back from the REST API are not always exactly
         # the same as the column header names.
         columns = [
-            'source_node', 'dest_node', 'source_compute', 'dest_compute',
-            'dest_host', 'status', 'instance_uuid', 'old_instance_type_id',
-            'new_instance_type_id', 'created_at', 'updated_at',
+            'source_node',
+            'dest_node',
+            'source_compute',
+            'dest_compute',
+            'dest_host',
+            'status',
+            'server_id',
+            'old_flavor_id',
+            'new_flavor_id',
+            'created_at',
+            'updated_at',
         ]
 
         # Insert migrations UUID after ID
-        if compute_client.api_version >= api_versions.APIVersion("2.59"):
+        if sdk_utils.supports_microversion(compute_client, "2.59"):
             column_headers.insert(0, "UUID")
             columns.insert(0, "uuid")
 
-        if compute_client.api_version >= api_versions.APIVersion("2.23"):
+        if sdk_utils.supports_microversion(compute_client, "2.23"):
             column_headers.insert(0, "Id")
             columns.insert(0, "id")
             column_headers.insert(len(column_headers) - 2, "Type")
             columns.insert(len(columns) - 2, "migration_type")
 
-        if compute_client.api_version >= api_versions.APIVersion("2.80"):
+        if sdk_utils.supports_microversion(compute_client, "2.80"):
             if parsed_args.project:
                 column_headers.insert(len(column_headers) - 2, "Project")
                 columns.insert(len(columns) - 2, "project_id")
@@ -159,19 +154,22 @@ class ListMigration(command.Lister):
         )
 
     def take_action(self, parsed_args):
-        compute_client = self.app.client_manager.compute
+        compute_client = self.app.client_manager.sdk_connection.compute
         identity_client = self.app.client_manager.identity
 
-        search_opts = {
-            'host': parsed_args.host,
-            'status': parsed_args.status,
-        }
+        search_opts = {}
+
+        if parsed_args.host is not None:
+            search_opts['host'] = parsed_args.host
+
+        if parsed_args.status is not None:
+            search_opts['status'] = parsed_args.status
 
         if parsed_args.server:
-            search_opts['instance_uuid'] = utils.find_resource(
-                compute_client.servers,
-                parsed_args.server,
-            ).id
+            server = compute_client.find_server(
+                parsed_args.server, ignore_missing=False
+            )
+            search_opts['instance_uuid'] = server.id
 
         if parsed_args.type:
             migration_type = parsed_args.type
@@ -181,7 +179,7 @@ class ListMigration(command.Lister):
             search_opts['migration_type'] = migration_type
 
         if parsed_args.marker:
-            if compute_client.api_version < api_versions.APIVersion('2.59'):
+            if not sdk_utils.supports_microversion(compute_client, "2.59"):
                 msg = _(
                     '--os-compute-api-version 2.59 or greater is required to '
                     'support the --marker option'
@@ -190,16 +188,17 @@ class ListMigration(command.Lister):
             search_opts['marker'] = parsed_args.marker
 
         if parsed_args.limit:
-            if compute_client.api_version < api_versions.APIVersion('2.59'):
+            if not sdk_utils.supports_microversion(compute_client, "2.59"):
                 msg = _(
                     '--os-compute-api-version 2.59 or greater is required to '
                     'support the --limit option'
                 )
                 raise exceptions.CommandError(msg)
             search_opts['limit'] = parsed_args.limit
+            search_opts['paginated'] = False
 
         if parsed_args.changes_since:
-            if compute_client.api_version < api_versions.APIVersion('2.59'):
+            if not sdk_utils.supports_microversion(compute_client, "2.59"):
                 msg = _(
                     '--os-compute-api-version 2.59 or greater is required to '
                     'support the --changes-since option'
@@ -208,7 +207,7 @@ class ListMigration(command.Lister):
             search_opts['changes_since'] = parsed_args.changes_since
 
         if parsed_args.changes_before:
-            if compute_client.api_version < api_versions.APIVersion('2.66'):
+            if not sdk_utils.supports_microversion(compute_client, "2.66"):
                 msg = _(
                     '--os-compute-api-version 2.66 or greater is required to '
                     'support the --changes-before option'
@@ -217,7 +216,7 @@ class ListMigration(command.Lister):
             search_opts['changes_before'] = parsed_args.changes_before
 
         if parsed_args.project:
-            if compute_client.api_version < api_versions.APIVersion('2.80'):
+            if not sdk_utils.supports_microversion(compute_client, "2.80"):
                 msg = _(
                     '--os-compute-api-version 2.80 or greater is required to '
                     'support the --project option'
@@ -231,7 +230,7 @@ class ListMigration(command.Lister):
             ).id
 
         if parsed_args.user:
-            if compute_client.api_version < api_versions.APIVersion('2.80'):
+            if not sdk_utils.supports_microversion(compute_client, "2.80"):
                 msg = _(
                     '--os-compute-api-version 2.80 or greater is required to '
                     'support the --user option'
@@ -244,20 +243,17 @@ class ListMigration(command.Lister):
                 parsed_args.user_domain,
             ).id
 
-        migrations = compute_client.migrations.list(**search_opts)
+        migrations = list(compute_client.migrations(**search_opts))
 
         return self.print_migrations(parsed_args, compute_client, migrations)
 
 
 def _get_migration_by_uuid(compute_client, server_id, migration_uuid):
-    for migration in compute_client.server_migrations.list(server_id):
+    for migration in compute_client.server_migrations(server_id):
         if migration.uuid == migration_uuid:
             return migration
-            break
     else:
-        msg = _(
-            'In-progress live migration %s is not found for server %s.'
-        )
+        msg = _('In-progress live migration %s is not found for server %s.')
         raise exceptions.CommandError(msg % (migration_uuid, server_id))
 
 
@@ -284,9 +280,9 @@ class ShowMigration(command.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
-        compute_client = self.app.client_manager.compute
+        compute_client = self.app.client_manager.sdk_connection.compute
 
-        if compute_client.api_version < api_versions.APIVersion('2.24'):
+        if not sdk_utils.supports_microversion(compute_client, '2.24'):
             msg = _(
                 '--os-compute-api-version 2.24 or greater is required to '
                 'support the server migration show command'
@@ -297,21 +293,19 @@ class ShowMigration(command.ShowOne):
             try:
                 uuid.UUID(parsed_args.migration)
             except ValueError:
-                msg = _(
-                    'The <migration> argument must be an ID or UUID'
-                )
+                msg = _('The <migration> argument must be an ID or UUID')
                 raise exceptions.CommandError(msg)
 
-            if compute_client.api_version < api_versions.APIVersion('2.59'):
+            if not sdk_utils.supports_microversion(compute_client, '2.59'):
                 msg = _(
                     '--os-compute-api-version 2.59 or greater is required to '
                     'retrieve server migrations by UUID'
                 )
                 raise exceptions.CommandError(msg)
 
-        server = utils.find_resource(
-            compute_client.servers,
+        server = compute_client.find_server(
             parsed_args.server,
+            ignore_missing=False,
         )
 
         # the nova API doesn't currently allow retrieval by UUID but it's a
@@ -319,14 +313,18 @@ class ShowMigration(command.ShowOne):
         # migrations - the responses are identical
         if not parsed_args.migration.isdigit():
             server_migration = _get_migration_by_uuid(
-                compute_client, server.id, parsed_args.migration,
+                compute_client,
+                server.id,
+                parsed_args.migration,
             )
         else:
-            server_migration = compute_client.server_migrations.get(
-                server.id, parsed_args.migration,
+            server_migration = compute_client.get_server_migration(
+                server.id,
+                parsed_args.migration,
+                ignore_missing=False,
             )
 
-        columns = (
+        column_headers = (
             'ID',
             'Server UUID',
             'Status',
@@ -345,14 +343,35 @@ class ShowMigration(command.ShowOne):
             'Updated At',
         )
 
-        if compute_client.api_version >= api_versions.APIVersion('2.59'):
-            columns += ('UUID',)
+        columns = (
+            'id',
+            'server_id',
+            'status',
+            'source_compute',
+            'source_node',
+            'dest_compute',
+            'dest_host',
+            'dest_node',
+            'memory_total_bytes',
+            'memory_processed_bytes',
+            'memory_remaining_bytes',
+            'disk_total_bytes',
+            'disk_processed_bytes',
+            'disk_remaining_bytes',
+            'created_at',
+            'updated_at',
+        )
 
-        if compute_client.api_version >= api_versions.APIVersion('2.80'):
-            columns += ('User ID', 'Project ID')
+        if sdk_utils.supports_microversion(compute_client, '2.59'):
+            column_headers += ('UUID',)
+            columns += ('uuid',)
+
+        if sdk_utils.supports_microversion(compute_client, '2.80'):
+            column_headers += ('User ID', 'Project ID')
+            columns += ('user_id', 'project_id')
 
         data = utils.get_item_properties(server_migration, columns)
-        return columns, data
+        return column_headers, data
 
 
 class AbortMigration(command.Command):
@@ -362,7 +381,7 @@ class AbortMigration(command.Command):
     """
 
     def get_parser(self, prog_name):
-        parser = super(AbortMigration, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
         parser.add_argument(
             'server',
             metavar='<server>',
@@ -376,9 +395,9 @@ class AbortMigration(command.Command):
         return parser
 
     def take_action(self, parsed_args):
-        compute_client = self.app.client_manager.compute
+        compute_client = self.app.client_manager.sdk_connection.compute
 
-        if compute_client.api_version < api_versions.APIVersion('2.24'):
+        if not sdk_utils.supports_microversion(compute_client, '2.24'):
             msg = _(
                 '--os-compute-api-version 2.24 or greater is required to '
                 'support the server migration abort command'
@@ -389,21 +408,19 @@ class AbortMigration(command.Command):
             try:
                 uuid.UUID(parsed_args.migration)
             except ValueError:
-                msg = _(
-                    'The <migration> argument must be an ID or UUID'
-                )
+                msg = _('The <migration> argument must be an ID or UUID')
                 raise exceptions.CommandError(msg)
 
-            if compute_client.api_version < api_versions.APIVersion('2.59'):
+            if not sdk_utils.supports_microversion(compute_client, '2.59'):
                 msg = _(
                     '--os-compute-api-version 2.59 or greater is required to '
                     'abort server migrations by UUID'
                 )
                 raise exceptions.CommandError(msg)
 
-        server = utils.find_resource(
-            compute_client.servers,
+        server = compute_client.find_server(
             parsed_args.server,
+            ignore_missing=False,
         )
 
         # the nova API doesn't currently allow retrieval by UUID but it's a
@@ -412,11 +429,15 @@ class AbortMigration(command.Command):
         migration_id = parsed_args.migration
         if not parsed_args.migration.isdigit():
             migration_id = _get_migration_by_uuid(
-                compute_client, server.id, parsed_args.migration,
+                compute_client,
+                server.id,
+                parsed_args.migration,
             ).id
 
-        compute_client.server_migrations.live_migration_abort(
-            server.id, migration_id,
+        compute_client.abort_server_migration(
+            migration_id,
+            server.id,
+            ignore_missing=False,
         )
 
 
@@ -427,23 +448,21 @@ class ForceCompleteMigration(command.Command):
     """
 
     def get_parser(self, prog_name):
-        parser = super(ForceCompleteMigration, self).get_parser(prog_name)
+        parser = super().get_parser(prog_name)
         parser.add_argument(
             'server',
             metavar='<server>',
             help=_('Server (name or ID)'),
         )
         parser.add_argument(
-            'migration',
-            metavar='<migration>',
-            help=_('Migration (ID)')
+            'migration', metavar='<migration>', help=_('Migration (ID)')
         )
         return parser
 
     def take_action(self, parsed_args):
-        compute_client = self.app.client_manager.compute
+        compute_client = self.app.client_manager.sdk_connection.compute
 
-        if compute_client.api_version < api_versions.APIVersion('2.22'):
+        if not sdk_utils.supports_microversion(compute_client, '2.22'):
             msg = _(
                 '--os-compute-api-version 2.22 or greater is required to '
                 'support the server migration force complete command'
@@ -454,21 +473,19 @@ class ForceCompleteMigration(command.Command):
             try:
                 uuid.UUID(parsed_args.migration)
             except ValueError:
-                msg = _(
-                    'The <migration> argument must be an ID or UUID'
-                )
+                msg = _('The <migration> argument must be an ID or UUID')
                 raise exceptions.CommandError(msg)
 
-            if compute_client.api_version < api_versions.APIVersion('2.59'):
+            if not sdk_utils.supports_microversion(compute_client, '2.59'):
                 msg = _(
                     '--os-compute-api-version 2.59 or greater is required to '
                     'abort server migrations by UUID'
                 )
                 raise exceptions.CommandError(msg)
 
-        server = utils.find_resource(
-            compute_client.servers,
+        server = compute_client.find_server(
             parsed_args.server,
+            ignore_missing=False,
         )
 
         # the nova API doesn't currently allow retrieval by UUID but it's a
@@ -477,9 +494,9 @@ class ForceCompleteMigration(command.Command):
         migration_id = parsed_args.migration
         if not parsed_args.migration.isdigit():
             migration_id = _get_migration_by_uuid(
-                compute_client, server.id, parsed_args.migration,
+                compute_client,
+                server.id,
+                parsed_args.migration,
             ).id
 
-        compute_client.server_migrations.live_migrate_force_complete(
-            server.id, migration_id,
-        )
+        compute_client.force_complete_server_migration(migration_id, server.id)
